/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "gimageview.h"

#include "gimv_image.h"
#include "gtk2-compat.h"
#include "gtkutils.h"
#include "menu.h"
#include "prefs.h"
#include "prefs_win.h"
#include "prefs_ui_imagewin.h"


typedef struct PrefsWin_Tag {
   GtkWidget *imgwin_width_spin;
   GtkWidget *imgwin_height_spin;
   GtkWidget *imgwin_menubar_toggle;
   GtkWidget *imgwin_toolbar_toggle;
   GtkWidget *imgwin_player_toggle;
   GtkWidget *imgwin_statusbar_toggle;
   GtkWidget *imgwin_scrollbar_toggle;
   GtkWidget *image_scale_spin;
} PrefsWin;


static PrefsWin prefs_win;
extern Config   *config_changed;
extern Config   *config_prechanged;


static void
set_sensitive_imgwin_state ()
{
   GtkWidget *widgets[] = {
      prefs_win.imgwin_width_spin,
      prefs_win.imgwin_height_spin,
      prefs_win.imgwin_menubar_toggle,
      prefs_win.imgwin_toolbar_toggle,
      prefs_win.imgwin_player_toggle,
      prefs_win.imgwin_statusbar_toggle,
      prefs_win.imgwin_scrollbar_toggle,
   };
   gint i, num = sizeof (widgets) / sizeof (widgets[0]);

   for (i = 0; i < num; i++)
      gtk_widget_set_sensitive (widgets[i],
				!config_changed->imgwin_save_win_state);
}


static void
cb_save_imgwin_state (GtkWidget *toggle)
{
   config_changed->imgwin_save_win_state =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(toggle));

   set_sensitive_imgwin_state ();
}


static void
cb_fit_image_to_win (GtkWidget *toggle)
{
   config_changed->imgview_fit_image_to_win =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(toggle));
   gtk_widget_set_sensitive (prefs_win.image_scale_spin,
			     !config_changed->imgview_fit_image_to_win);
}


static const gchar *imageview_mouse_items[] = {
   N_("None"),
   N_("Next image"),
   N_("Previous image"),
   N_("Popup menu"),
   N_("Zoom in"),
   N_("Zoom out"),
   N_("Fit image size to frame"),
   N_("Rotate CCW"),
   N_("Rotate CW"),
   N_("Open navigate window"),
   NULL
};


/*******************************************************************************
 *  prefs_imageview_page:
 *     @ Create Image View preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_imageview_page ()
{
   GtkWidget *main_vbox;
   GtkWidget *frame;
   GtkWidget *hbox, *vbox;
   GtkWidget *label;
   GtkAdjustment *adj;
   GtkWidget *spinner;
   GtkWidget *toggle;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    * Window Frame
    **********************************************/
   prefs_create_frame(_("Window"), frame, vbox, main_vbox);

   /* Save window state on exit */
   toggle = create_check_button (_("Save window state on exit"),
				 conf.imgwin_save_win_state,
                                 cb_save_imgwin_state,
                                 &config_changed->imgwin_save_win_state);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Window width and height spinner */
   hbox = gtk_hbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

   label = gtk_label_new (_("Initial window size: "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

   label = gtk_label_new (_("width"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.imgwin_width,
					       1.0, 10000.0, 1.0, 5.0, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   prefs_win.imgwin_width_spin = spinner;
   gtk_widget_set_usize(spinner, 50, -1);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinner), GTK_SHADOW_OUT);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		       GTK_SIGNAL_FUNC (cb_get_data_from_adjustment_by_int),
		       &config_changed->imgwin_width);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);

   label = gtk_label_new (_("height"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.imgwin_height,
					       1.0, 10000.0, 1.0, 5.0, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   prefs_win.imgwin_height_spin = spinner;
   gtk_widget_set_usize(spinner, 50, -1);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinner), GTK_SHADOW_OUT);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		       GTK_SIGNAL_FUNC (cb_get_data_from_adjustment_by_int),
		       &config_changed->imgwin_height);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);

   /* Auto resize window to image size */
   toggle = create_check_button (_("Auto resize to image size"),
				 conf.imgwin_fit_to_image,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_fit_to_image);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Open New Window or not */
   toggle = create_check_button (_("Open each file in separate window"),
				 conf.imgwin_open_new_win,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_open_new_win);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Raise window or not */
   toggle = create_check_button (_("Raise window when open image in shared window"),
				 conf.imgwin_raise_window,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_raise_window);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /********************************************** 
    * Show/Hide Frame
    **********************************************/
   prefs_create_frame(_("Show/Hide"), frame, vbox, main_vbox);

   /* Show Menubar or not */
   toggle = create_check_button (_("Show menubar"), conf.imgwin_show_menubar,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_show_menubar);
   prefs_win.imgwin_menubar_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Toolbar or not */
   toggle = create_check_button (_("Show toolbar"), conf.imgwin_show_toolbar,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_show_toolbar);
   prefs_win.imgwin_toolbar_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Player toolbar or not */
   toggle = create_check_button (_("Show player toolbar"), conf.imgwin_show_player,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_show_player);
   prefs_win.imgwin_player_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Statusbar or not */
   toggle = create_check_button (_("Show statusbar"), conf.imgwin_show_statusbar,
				 cb_get_data_from_toggle,
				 &config_changed->imgwin_show_statusbar);
   prefs_win.imgwin_statusbar_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Scrollbar or not */
   toggle = create_check_button (_("Show scrollbar"), conf.imgview_scrollbar,
				 cb_get_data_from_toggle,
				 &config_changed->imgview_scrollbar);
   prefs_win.imgwin_scrollbar_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /********************************************** 
    * Image Frame
    **********************************************/
   prefs_create_frame(_("Image"), frame, vbox, main_vbox);

   /* Auto resize image to window size */
   toggle = create_check_button (_("Auto resize to window size when opening"),
				 conf.imgview_fit_image_to_win,
				 cb_fit_image_to_win, NULL);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Keep Aspect Ratio */
   toggle = create_check_button (_("Keep aspect rario"), conf.imgview_keep_aspect,
				 cb_get_data_from_toggle,
				 &config_changed->imgview_keep_aspect);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Default Image Scale Spinner */
   hbox = gtk_hbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new (_("Default Image Scale"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.imgview_scale,
					       1.0, 10000.0, 1.0, 5.0, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   gtk_widget_set_usize(spinner, 50, -1);
   prefs_win.image_scale_spin = spinner;
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinner), GTK_SHADOW_OUT);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		       GTK_SIGNAL_FUNC (cb_get_data_from_adjustment_by_int),
		       &config_changed->imgview_scale);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);
   label = gtk_label_new (_("%"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   /*
   gtk_widget_set_sensitive (prefs_win.image_scale_spin,
			     !conf.imgview_fit_image_to_win);
   */

   /* keep original image on memory or not */
   toggle = create_check_button (_("Keep original image on memory"),
				 conf.imgview_buffer,
				 cb_get_data_from_toggle,
				 &config_changed->imgview_buffer);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_preview_mouse_page:
 *     @ Create mouse button action on preview preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_imageview_mouse_page ()
{
   GtkWidget *vbox;

   vbox = create_mouse_prefs (imageview_mouse_items,
			      conf.imgview_mouse_button,
			      &config_changed->imgview_mouse_button);

   return vbox;
}


void
prefs_ui_imagewin_set_sensitive ()
{
   set_sensitive_imgwin_state ();
   gtk_widget_set_sensitive (prefs_win.image_scale_spin,
			     !conf.imgview_fit_image_to_win);
}
