/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "gimageview.h"

#include "gtkutils.h"
#include "prefs.h"
#include "prefs_ui_progs.h"
#include "prefs_win.h"


typedef struct PrefsWin_Tag {
   /* external progs */
   GtkWidget *graph_progs_clist;
   GtkWidget *graph_progs_name;
   GtkWidget *graph_progs_command;
   GtkWidget *graph_progs_dialog_toggle;
   GtkWidget *graph_progs_up;
   GtkWidget *graph_progs_down;
   GtkWidget *graph_progs_new;
   GtkWidget *graph_progs_add;
   GtkWidget *graph_progs_change;
   GtkWidget *graph_progs_delete;
   gint       graph_progs_selected;

   GtkWidget *text_viewer_command;

   /* scripts */
   GtkWidget *scripts_clist;
   GtkWidget *scripts_entry;
   GtkWidget *scripts_up;
   GtkWidget *scripts_down;
   GtkWidget *scripts_new;
   GtkWidget *scripts_add;
   GtkWidget *scripts_change;
   GtkWidget *scripts_delete;
   gint       scripts_selected;
} PrefsWin;

static PrefsWin  prefs_win;
extern Config   *config_changed;
extern Config   *config_prechanged;
static void


set_sensitive_ext_progs (gboolean text_changed)
{
   const gchar *text[2];
   gint num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint rownum = GTK_CLIST (prefs_win.graph_progs_clist)->rows;
   gint selected = prefs_win.graph_progs_selected;

   text[0] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_name));
   text[1] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_command));

   if (selected < 0) {   /* no item selected */
      gtk_widget_set_sensitive (prefs_win.graph_progs_up, FALSE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_down, FALSE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_change, FALSE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_delete, FALSE);
   } else {   /* if an item selected */
      /* change button */
      if (text_changed)
	 gtk_widget_set_sensitive (prefs_win.graph_progs_change, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.graph_progs_change, FALSE);

      /* up button */
      if (selected == 0)
	 gtk_widget_set_sensitive (prefs_win.graph_progs_up, FALSE);
      else
	 gtk_widget_set_sensitive (prefs_win.graph_progs_up, TRUE);

      /* down button */
      if (selected < rownum - 1)
	 gtk_widget_set_sensitive (prefs_win.graph_progs_down, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.graph_progs_down, FALSE);

      /* delete button */
      gtk_widget_set_sensitive (prefs_win.graph_progs_delete, TRUE);
   }

   /* add button */
   if (rownum >= num
       || !text[0] || text[0][0] == '\0'
       || !text[1] || text[1][0] == '\0' )
   {
      if (selected < 0)
	 gtk_widget_set_sensitive (prefs_win.graph_progs_new, FALSE);
      else
	 gtk_widget_set_sensitive (prefs_win.graph_progs_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_add, FALSE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_change, FALSE);
  } else {
      gtk_widget_set_sensitive (prefs_win.graph_progs_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.graph_progs_add, TRUE);
      if (selected >= 0)
	 gtk_widget_set_sensitive (prefs_win.graph_progs_change, TRUE);
  }
}


static void
cb_progs_select_row (GtkCList *clist, gint row, gint col,
		     GdkEventButton *event, gpointer data)
{
   gint success;
   gchar *text[3] = {NULL, NULL, NULL};
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.graph_progs_dialog_toggle);

   g_return_if_fail (clist);

   prefs_win.graph_progs_selected = row;

   success = gtk_clist_get_text (clist, row, 0, &text[0]);
   if (success && text[0])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_name), text[0]);

   success = gtk_clist_get_text (clist, row, 1, &text[1]);
   if (success && text[1])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_command), text[1]);

   success = gtk_clist_get_text (clist, row, 2, &text[2]);
   if (success && text[2]
       && (!g_strcasecmp ("TRUE", text[2])
	   || !g_strcasecmp (_("TRUE"), text[2])))
   {
      gtk_toggle_button_set_active (toggle, TRUE);
   } else {
      gtk_toggle_button_set_active (toggle, FALSE);
   }

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_unselect_row (GtkCList *clist, gint row, gint col,
		       GdkEventButton *event, gpointer data)
{
   g_return_if_fail (clist);

   prefs_win.graph_progs_selected = -1;

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_up_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;
   gint selected = prefs_win.graph_progs_selected;
   gint num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint rownum = GTK_CLIST (prefs_win.graph_progs_clist)->rows;
   gchar * swap;

   g_return_if_fail (button && clist);

   if (selected < 1 || selected > rownum - 1
       || selected > num - 1)
   {
      return;
   }

   swap = config_changed->progs[selected];
   config_changed->progs[selected] = config_changed->progs[selected - 1];
   config_changed->progs[selected - 1] = swap;
   gtk_clist_swap_rows (clist, selected, selected - 1);
   prefs_win.graph_progs_selected--;

   gtk_clist_moveto (clist, selected - 1, 0, 0, 0);

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_down_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;
   gint selected = prefs_win.graph_progs_selected;
   gint num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint rownum = GTK_CLIST (prefs_win.graph_progs_clist)->rows;
   gchar * swap;

   g_return_if_fail (button && clist);

   if (selected < 0 || selected > rownum - 2
       || selected > num - 2)
   {
      return;
   }

   swap = config_changed->progs[selected];
   config_changed->progs[selected] = config_changed->progs[selected + 1];
   config_changed->progs[selected + 1] = swap;
   gtk_clist_swap_rows (clist, selected, selected + 1);
   prefs_win.graph_progs_selected++;

   gtk_clist_moveto (clist, selected + 1, 0, 0, 0);

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_new_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;

   g_return_if_fail (button && data);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_name), "\0");
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_command), "\0");

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.graph_progs_dialog_toggle);
   const gchar *text[16];
   gint num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint rownum = GTK_CLIST (prefs_win.graph_progs_clist)->rows;
   gboolean show_dialog;

   g_return_if_fail (clist);

   text[0] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_name));
   text[1] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_command));
   show_dialog = gtk_toggle_button_get_active (toggle);
   if (show_dialog)
      text[2] = "TRUE";
   else
      text[2] = "FALSE";

   if (text[0] && text[1] && rownum < num) {
      config_changed->progs[rownum]
	 = g_strconcat (text[0], ",", text[1], ",", text[2], NULL);
      text[2] = _(text[2]);
      gtk_clist_append (GTK_CLIST (clist), (gchar **) text);
   }

   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_name), "\0");
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_command), "\0");
   gtk_toggle_button_set_active (toggle, FALSE);

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_change_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.graph_progs_dialog_toggle);
   gchar *src;
   const gchar *text[16];
   gint num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint selected = prefs_win.graph_progs_selected;
   gboolean show_dialog;

   g_return_if_fail (clist);

   if (selected < 0 && selected > num) return;

   text[0] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_name));
   text[1] = gtk_entry_get_text (GTK_ENTRY (prefs_win.graph_progs_command));
   show_dialog = gtk_toggle_button_get_active (toggle);
   if (show_dialog)
      text[2] = "TRUE";
   else
      text[2] = "FALSE";

   if (text[0] && text[1]) {
      src = gtk_clist_get_row_data (clist, selected);
      if (!src) {
	 g_free (config_changed->progs[selected]);
	 gtk_clist_set_row_data (clist, selected, NULL);
      }
      config_changed->progs[selected]
	 = g_strconcat (text[0], ",", text[1], ",", text[2], NULL);
      gtk_clist_set_text (clist, selected, 0, text[0]);
      gtk_clist_set_text (clist, selected, 1, text[1]);
      gtk_clist_set_text (clist, selected, 2, _(text[2]));
   }

   /*
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_name), "\0");
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_command), "\0");
   */

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_delete_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = data;
   gchar *src;
   gint i, num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gint selected = prefs_win.graph_progs_selected;

   g_return_if_fail (clist);

   if (selected < 0 && selected > num) return;

   src = gtk_clist_get_row_data (clist, selected);

   if (!src)
      g_free (config_changed->progs[selected]);
   config_changed->progs[selected] = NULL;

   for (i = selected; i < num - 1; i++) {
      config_changed->progs[i] = config_changed->progs[i + 1];
   }
   config_changed->progs[num] = NULL;

   gtk_clist_remove (clist, selected);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_name), "\0");
   gtk_entry_set_text (GTK_ENTRY(prefs_win.graph_progs_command), "\0");

   set_sensitive_ext_progs (FALSE);
}


static void
cb_progs_entry_changed (GtkEditable *entry, gpointer data)
{
   set_sensitive_ext_progs (TRUE);
}


static void
cb_use_internal_text_viewer (GtkWidget *toggle)
{
   config_changed->text_viewer_use_internal =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(toggle));

   gtk_widget_set_sensitive (prefs_win.text_viewer_command,
			     !config_changed->text_viewer_use_internal);
}


static void
cb_text_view_command_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->text_viewer != config_prechanged->text_viewer)
      g_free (config_changed->text_viewer);

   config_changed->text_viewer
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
cb_web_browser_command_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->web_browser != config_prechanged->web_browser)
      g_free (config_changed->web_browser);

   config_changed->web_browser
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
scripts_update_config ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   gint row, can_retrieve = 1;
   gchar *coltext, *string = NULL, *tmpstr;

   for (row = 0; row < clist->rows; row++) {
      can_retrieve = gtk_clist_get_text (clist, row, 0, &coltext);
      if (!can_retrieve) break;

      if (string) {
	 tmpstr = string;
	 string = g_strconcat (string, ",", NULL);
	 g_free (tmpstr);
      } else {
	 string = g_strdup ("");
      }
      tmpstr = string;
      string = g_strconcat (tmpstr, coltext, NULL);
      g_free (tmpstr);
   }

   if (config_prechanged->scripts_search_dir_list
       != config_changed->scripts_search_dir_list)
   {
      g_free (config_changed->scripts_search_dir_list);
   }

   config_changed->scripts_search_dir_list = string;
}


static void
set_sensitive_scripts (gboolean text_changed)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.scripts_entry);
   gint selected = prefs_win.scripts_selected;
   const gchar *text;

   text = gtk_entry_get_text (entry);

   if (selected < 0) {   /* no item selected */
      gtk_widget_set_sensitive (prefs_win.scripts_up, FALSE);
      gtk_widget_set_sensitive (prefs_win.scripts_down, FALSE);
      gtk_widget_set_sensitive (prefs_win.scripts_change, FALSE);
      gtk_widget_set_sensitive (prefs_win.scripts_delete, FALSE);
   } else {
      /* up button */
      if (selected == 0)
	 gtk_widget_set_sensitive (prefs_win.scripts_up, FALSE);
      else
	 gtk_widget_set_sensitive (prefs_win.scripts_up, TRUE);

      /* down button */
      if (selected < clist->rows - 1)
	 gtk_widget_set_sensitive (prefs_win.scripts_down, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.scripts_down, FALSE);

      /* change button */
      if (text_changed)
	 gtk_widget_set_sensitive (prefs_win.scripts_change, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.scripts_change, FALSE);

      /* delete button */
      gtk_widget_set_sensitive (prefs_win.scripts_delete, TRUE);
   }

   if (!text || *text == '\0')
   {
      gtk_widget_set_sensitive (prefs_win.scripts_new, FALSE);
      gtk_widget_set_sensitive (prefs_win.scripts_add, FALSE);
      gtk_widget_set_sensitive (prefs_win.scripts_change, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.scripts_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.scripts_add, TRUE);
      if (selected >= 0)
	 gtk_widget_set_sensitive (prefs_win.scripts_change, TRUE);
   }
}


static void
cb_scripts_select_row (GtkCList *clist, gint row, gint col,
		       GdkEventButton *event, gpointer data)
{
   gint success;
   gchar *text[1] = {NULL};

   g_return_if_fail (clist);

   prefs_win.scripts_selected = row;

   success = gtk_clist_get_text (clist, row, 0, &text[0]);
   if (success && text[0])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.scripts_entry), text[0]);

   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_unselect_row (GtkCList *clist, gint row, gint col,
			 GdkEventButton *event, gpointer data)
{
   g_return_if_fail (clist);

   prefs_win.scripts_selected = -1;

   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_up_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   gint selected = prefs_win.scripts_selected;

   g_return_if_fail (button && clist);

   if (selected < 1 || selected > clist->rows - 1)
   {
      return;
   }

   gtk_clist_swap_rows (clist, selected, selected - 1);
   gtk_clist_moveto (clist, selected - 1, 0, 0, 0);
   prefs_win.scripts_selected--;

   scripts_update_config ();
   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_down_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   gint selected = prefs_win.scripts_selected;

   g_return_if_fail (button && clist);

   if (selected < 0 || selected > clist->rows - 2)
   {
      return;
   }

   gtk_clist_swap_rows (clist, selected, selected + 1);
   gtk_clist_moveto (clist, selected + 1, 0, 0, 0);
   prefs_win.scripts_selected++;

   scripts_update_config ();
   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_new_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.scripts_entry);

   g_return_if_fail (button);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (entry, "\0");

   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.scripts_entry);
   const gchar *text[1];

   g_return_if_fail (button);

   text[0] = gtk_entry_get_text (entry);
   gtk_clist_append (clist, (gchar **) text);
   gtk_entry_set_text (entry, "\0");

   scripts_update_config ();
   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_change_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.scripts_entry);
   const gchar *text[1];
   gint selected = prefs_win.scripts_selected;

   if (selected < 0 && selected > clist->rows) return;

   text[0] = gtk_entry_get_text (entry);
   gtk_clist_set_text (clist, selected, 0, text[0]);
   /* gtk_entry_set_text (entry, "\0"); */

   scripts_update_config ();
   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_delete_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.scripts_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.scripts_entry);
   gint selected = prefs_win.scripts_selected;

   if (selected < 0 && selected > clist->rows) return;

   gtk_clist_remove (clist, selected);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (entry, "\0");

   scripts_update_config ();
   set_sensitive_scripts (FALSE);
}


static void
cb_scripts_entry_changed (GtkEditable *entry, gpointer data)
{
   set_sensitive_scripts (TRUE);
}


/*******************************************************************************
 *  prefs_progs_page:
 *     @ Create external programs preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_progs_page ()
{
   gchar *titles[] = {
      N_("Program Name"),
      N_("Command"),
      N_("Dialog"),
   };

   GtkWidget *main_vbox, *frame, *toggle;
   GtkWidget *frame_vbox, *vbox, *hbox, *hbox1, *hbox2, *scrollwin;
   GtkWidget *clist, *label, *entry, *button;
   gchar *data[16], **pair;
   gint i, j, num;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Viewer/Editor frame
    **********************************************/
   prefs_create_frame (_("Graphic Viewer/Editor"), frame, frame_vbox, main_vbox);

   /* clist */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 120);

   num = sizeof (titles) / sizeof (gchar *);
   clist = prefs_win.graph_progs_clist = gtk_clist_new (num);
   for (i = 0; i < num; i++) {
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
      gtk_clist_set_column_auto_resize (GTK_CLIST (clist), i, TRUE);
   }
   gtk_clist_column_titles_show (GTK_CLIST (clist));
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);

   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_progs_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_progs_unselect_row),
		       NULL);

   /* set default values */
   num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   j = 0;
   for (i = 0; i < num; i++) {
      if (config_changed->progs[i]) {
	 pair = g_strsplit (conf.progs[i], ",", 3);
	 if (pair[0] && pair[1]) {
	    data[0] = pair[0];
	    data[1] = pair[1];
	    if (pair[2] && !strcasecmp ("TRUE", pair[2]))
	       data[2] = _("TRUE");
	    else
	       data[2] = _("FALSE");
	    gtk_clist_append (GTK_CLIST (clist), data);
	    gtk_clist_set_row_data (GTK_CLIST (clist), j,
				    config_prechanged->progs[i]);
	    config_changed->progs[j] = config_changed->progs[i];
	    j++;
	 }
	 g_strfreev (pair);
      }
   }

   prefs_win.graph_progs_selected = -1;

   /* move buttons */
   vbox = gtk_vbox_new (TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

   button = gtk_button_new_with_label (_("Up"));
   prefs_win.graph_progs_up = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_up_button_pressed), clist);

   button = gtk_button_new_with_label (_("Down"));
   prefs_win.graph_progs_down = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_down_button_pressed), clist);

   /* entry area */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
   label = gtk_label_new (_("Program Name: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
   prefs_win.graph_progs_name = entry = gtk_entry_new ();
   gtk_widget_set_usize (entry, 100, -1);
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_progs_entry_changed), NULL);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
   label = gtk_label_new (_("Command: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);

   hbox2 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);
   prefs_win.graph_progs_command = entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox2), entry, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_progs_entry_changed), NULL);

   /* insert check box */
   toggle = create_check_button (_("Dialog"), FALSE, NULL, NULL);
   prefs_win.graph_progs_dialog_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (hbox2), toggle, FALSE, FALSE, 0);

   /* edit buttons */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox1, FALSE, TRUE, 0);
   gtk_box_set_homogeneous (GTK_BOX(hbox1), TRUE);

   button = gtk_button_new_with_label (_("New"));
   prefs_win.graph_progs_new = button;
   gtk_widget_set_usize (button, 70, -1);
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_new_button_pressed), clist);

   button = gtk_button_new_with_label (_("Add"));
   prefs_win.graph_progs_add = button;
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_add_button_pressed), clist);

   button = gtk_button_new_with_label (_("Change"));
   prefs_win.graph_progs_change = button;
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_change_button_pressed), clist);

   button = gtk_button_new_with_label (_("Delete"));
   prefs_win.graph_progs_delete = button;
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_progs_delete_button_pressed), clist);

   /* set_sensitive_ext_progs (FALSE); */

   /**********************************************
    *  Web Browser frame
    **********************************************/
   prefs_create_frame (_("Web Browser"), frame, frame_vbox, main_vbox);

   /* web browser command */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);
   label = gtk_label_new (_("Command: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   gtk_entry_set_text (GTK_ENTRY (entry), conf.web_browser);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_web_browser_command_changed), NULL);

   /**********************************************
    *  Text Viewer frame
    **********************************************/
   prefs_create_frame (_("Text Viewer"), frame, frame_vbox, main_vbox);

   /* use internal text viewer or not */
   toggle = create_check_button (_("Use internal viewer"),
				 conf.text_viewer_use_internal,
				 cb_use_internal_text_viewer,
				 &config_changed->text_viewer_use_internal);
   gtk_box_pack_start (GTK_BOX (frame_vbox), toggle, FALSE, FALSE, 0);

   /* text viewer command */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);
   label = gtk_label_new (_("Command: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   prefs_win.text_viewer_command = entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   gtk_entry_set_text (GTK_ENTRY (entry), conf.text_viewer);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_text_view_command_changed), NULL);

   /*
   gtk_widget_set_sensitive (prefs_win.text_viewer_command,
   			     !config_changed->text_viewer_use_internal);
   */

   return main_vbox;
}


/*******************************************************************************
 *  prefs_scripts_page:
 *     @ Create List View preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_scripts_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox, *hbox1, *vbox;
   GtkWidget *label, *entry, *scrollwin, *clist, *button, *toggle;
   gint i, num;
   gchar **dirs;
   gchar *titles[] = {
      N_("Directory"),
   };

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Search Directory frame
    **********************************************/
   prefs_create_frame (_("Directories list to search scripts"),
		       frame, frame_vbox, main_vbox);

   /* clist */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 150);

   num = sizeof (titles)/ sizeof (titles[0]);
   clist = prefs_win.scripts_clist = gtk_clist_new (num);
   gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
   for (i = 0; i < num; i++)
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
   gtk_clist_column_titles_hide (GTK_CLIST (clist));
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);
   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_scripts_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_scripts_unselect_row),
		       NULL);

   /* set default value */
   dirs = g_strsplit (conf.scripts_search_dir_list, ",", -1);
   if (dirs) {
      for (i = 0; dirs[i]; i++) {
	 if (*dirs[i])
	    gtk_clist_append (GTK_CLIST (clist), &dirs[i]);
      }
   }
   g_strfreev (dirs);

   /* move buttons */
   vbox = gtk_vbox_new (TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

   button = gtk_button_new_with_label (_("Up"));
   prefs_win.scripts_up = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_up_button_pressed), NULL);

   button = gtk_button_new_with_label (_("Down"));
   prefs_win.scripts_down = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_down_button_pressed), NULL);


   /* entry area */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   label = gtk_label_new (_("Directory to add : "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   entry = prefs_win.scripts_entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_scripts_entry_changed), NULL);

   /* edit buttons */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox1, FALSE, TRUE, 0);
   gtk_box_set_homogeneous (GTK_BOX(hbox1), TRUE);

   button = prefs_win.scripts_new =gtk_button_new_with_label (_("New"));
   gtk_widget_set_usize (button, 70, -1);
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_new_button_pressed), NULL);

   button = prefs_win.scripts_add = gtk_button_new_with_label (_("Add"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_add_button_pressed), NULL);

   button = prefs_win.scripts_change = gtk_button_new_with_label (_("Change"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_change_button_pressed), NULL);

   button = prefs_win.scripts_delete = gtk_button_new_with_label (_("Delete"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_scripts_delete_button_pressed), NULL);

   prefs_win.scripts_selected = -1;

   /* Show dialog */
   toggle = create_check_button (_("Show dialog befor execute script"),
				 conf.scripts_show_dialog,
				 cb_get_data_from_toggle,
				 &config_changed->scripts_show_dialog);
   gtk_box_pack_start (GTK_BOX (main_vbox), toggle, FALSE, FALSE, 0);

   return main_vbox;
}


void
prefs_ui_progs_set_sensitive ()
{
   set_sensitive_ext_progs (FALSE);
   gtk_widget_set_sensitive (prefs_win.text_viewer_command,
			     !config_changed->text_viewer_use_internal);
   set_sensitive_scripts (FALSE);
}
