/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>

#include "gimageview.h"

#include "comment.h"
#include "gedo-hpaned.h"
#include "gimv_image.h"
#include "gtk2-compat.h"
#include "gtkutils.h"
#include "icon.h"
#include "menu.h"
#include "plugin.h"
#include "prefs.h"
#include "prefs_ui_common.h"
#include "prefs_ui_etc.h"
#include "prefs_ui_plugin.h"
#include "prefs_ui_progs.h"
#include "prefs_ui_imagewin.h"
#include "prefs_ui_thumbwin.h"
#include "prefs_win.h"
#include "thumbnail_window.h"
#include "thumbview_detail.h"


typedef struct PrefsWin_Tag {
   GtkWidget *notebook;
   GtkWidget *ctree;
} PrefsWin;


static PrefsPage prefs_pages[] = {
   {N_("Common"),        NULL,     NULL, NULL, NULL, prefs_common_page,  NULL, NULL},
   {N_("Filtering"),     "Common", NULL, NULL, NULL, prefs_filter_page,  NULL, NULL},
   {N_("Character set"), "Common", NULL, NULL, NULL, prefs_charset_page, NULL, NULL},

   {N_("Image Window"),  NULL,           NULL, NULL, NULL, prefs_imageview_page,       NULL, NULL},
   {N_("Mouse Buttton"), "Image Window", NULL, NULL, NULL, prefs_imageview_mouse_page, NULL, NULL},

   {N_("Thumbnail Window"), NULL,               NULL, NULL, NULL, prefs_thumbwin_page, NULL, NULL},
   /*
   {N_("Toolbar"),          "Thumbnail Window", NULL, NULL, NULL, prefs_thumbview_toolbar_page, NULL, NULL},
   */
   {N_("Tab"),              "Thumbnail Window", NULL, NULL, NULL, prefs_thumbwin_tab_page,    NULL, NULL},
   {N_("Thumbnail View"),   "Thumbnail Window", NULL, NULL, NULL, prefs_thumbview_page,       NULL, NULL},
   {N_("Thumbnail Table"),  "Thumbnail View",   NULL, NULL, NULL, prefs_thumbtable_page,      NULL, NULL},
   {N_("List View"),        "Thumbnail View",   NULL, NULL, NULL, prefs_listview_page,        NULL, NULL},
   {N_("Detail View"),      "Thumbnail View",   NULL, NULL, NULL, prefs_detailview_page,      NULL, NULL},
   {N_("Mouse Buttton"),    "Thumbnail View",   NULL, NULL, NULL, prefs_thumbview_mouse_page, NULL, NULL},

   {N_("Directory View"),   "Thumbnail Window", NULL, NULL, NULL, prefs_dirview_page,       NULL, NULL},
   {N_("Preview"),          "Thumbnail Window", NULL, NULL, NULL, prefs_preview_page,       NULL, NULL},
   {N_("Mouse Buttton"),    "Preview",          NULL, NULL, NULL, prefs_preview_mouse_page, NULL, NULL},

   {N_("Slide Show"),       NULL,               NULL, NULL, NULL, prefs_slideshow_page, NULL, NULL},
   {N_("Thumbnail Cache"),  NULL,               NULL, NULL, NULL, prefs_cache_page,     NULL, NULL},
   {N_("Comment"),          NULL,               NULL, NULL, NULL, prefs_comment_page,   NULL, NULL},
   {N_("Search"),           NULL,               NULL, NULL, NULL, prefs_search_page,    NULL, NULL},
   {N_("Drag and Drop"),    NULL,               NULL, NULL, NULL, prefs_dnd_page,       NULL, NULL},
   {N_("External Program"), NULL,               NULL, NULL, NULL, prefs_progs_page,     NULL, NULL},
   {N_("Scripts"),          "External Program", NULL, NULL, NULL, prefs_scripts_page,   NULL, NULL},

   {N_("Plugin"),      NULL,     NULL, NULL, NULL, prefs_plugin_page,             NULL, NULL},
   {N_("Image Saver"), "Plugin", NULL, NULL, NULL, prefs_plugin_image_saver_page, NULL, NULL},
   {N_("Archiver"),    "Plugin", NULL, NULL, NULL, prefs_plugin_archiver_page,    NULL, NULL},
   {N_("Thumbnail"),   "Plugin", NULL, NULL, NULL, prefs_plugin_thumbnail_page,   NULL, NULL},
};
static gint prefs_pages_num = sizeof (prefs_pages) / sizeof (prefs_pages[0]);


Config    *config_changed    = NULL;
Config    *config_prechanged = NULL;
static GtkWidget *prefs_window;
static gboolean   prefs_ok_pressed  = FALSE;
static PrefsWin   prefs_win;


/*******************************************************************************
 *
 *   Callback functions for preference window.
 *
 *******************************************************************************/
/*
 *  conf_apply_to_existing_win:
 *     @ Apply changed config to existing window.
 *
 *  src  : source config data.
 *  dest : destination config data.
 */
static void
conf_apply_to_existing_win (Config *src, Config *dest)
{
   ThumbWindow *tw;
   GList *node;

   /* fileter */
   gimv_image_update_image_types ();

   /* charset */
   gimv_set_charset ();

   /* thumbnail window */
   node = g_list_first (ThumbWinList);
   while (node) {
      tw = node->data;
      gtk_toolbar_set_style (GTK_TOOLBAR(tw->toolbar),
			     dest->thumbwin_toolbar_style);

      gtk_widget_queue_resize (tw->toolbar_handle);

      if (tw->dv) {
	 gtk_ctree_set_line_style (GTK_CTREE (tw->dv->dirtree),
				   dest->dirview_line_style);
	 gtk_ctree_set_expander_style (GTK_CTREE (tw->dv->dirtree),
				       dest->dirview_expander_style);
      }

      node = node->next;
   }

   /* detail view */
   if (conf.detailview_columns != config_prechanged->detailview_columns)
      detailview_apply_config ();

   /* comment */
   comment_update_data_entry_list ();

   /* plugin */
   if (conf.plugin_search_dir_list != config_prechanged->plugin_search_dir_list)
      plugin_create_search_dir_list ();

   /* tab label */
   node = g_list_first (ThumbViewList);
   while (node) {
      ThumbView *tv = node->data;
      thumbview_reset_tab_label (tv);
      node = g_list_next (node);
   }
}


/* FIXME!! It is too stupid ;-( */
/* Damn Damn Damn XD */
static void
strings_free (gboolean value_changed)
{
   gint i, j;
   gchar *strings[13][2];
   gint num = sizeof (strings) / sizeof (strings[0]);
   gchar *progs_string1 = NULL, *progs_string2 = NULL;
   gint progs_num = sizeof (conf.progs) / sizeof (conf.progs[0]);
   gboolean not_free = FALSE;

#if 0
   {
      gint offset = (gchar *) &conf.text_viewer - (gchar *) &conf;
      gchar **str = (gchar *) config_changed + offset;
      g_free (*str);
      *str = NULL;
   }
#endif

   strings[0][0] = config_prechanged->text_viewer;
   strings[0][1] = config_changed->text_viewer;
   strings[1][0] = config_changed->detailview_columns;
   strings[1][1] = config_changed->detailview_columns;
   strings[2][0] = config_prechanged->web_browser;
   strings[2][1] = config_changed->web_browser;
   strings[3][0] = config_prechanged->plugin_search_dir_list;
   strings[3][1] = config_changed->plugin_search_dir_list;
   strings[4][0] = config_prechanged->scripts_search_dir_list;
   strings[4][1] = config_changed->scripts_search_dir_list;
   strings[5][0] = config_prechanged->imgview_mouse_button;
   strings[5][1] = config_changed->imgview_mouse_button;
   strings[6][0] = config_prechanged->preview_mouse_button;
   strings[6][1] = config_changed->preview_mouse_button;
   strings[7][0] = config_prechanged->listview_data_order;
   strings[7][1] = config_changed->listview_data_order;
   strings[8][0] = config_prechanged->imgtype_disables;
   strings[8][1] = config_changed->imgtype_disables;
   strings[9][0] = config_prechanged->imgtype_user_defs;
   strings[9][1] = config_changed->imgtype_user_defs;
   strings[10][0] = config_prechanged->comment_key_list;
   strings[10][1] = config_changed->comment_key_list;
   strings[11][0] = config_prechanged->charset_locale;
   strings[11][1] = config_changed->charset_locale;
   strings[12][0] = config_prechanged->charset_internal;
   strings[12][1] = config_changed->charset_internal;

   for (i = 0; i < num; i++) {
      if (strings[i][0] == strings[i][1]) continue;

      if (value_changed) {
	 g_free (strings[i][0]);
      } else {
	 g_free (strings[i][1]);
      }
   }

   /* free old external programs string */
   for (i = 0; i < progs_num; i++) {
      for (j = 0; j < progs_num; j++) {
	 if (value_changed) {
	    progs_string1 = config_prechanged->progs[i];
	    progs_string2 = config_changed->progs[j];
	 } else {
	    progs_string1 = config_changed->progs[i];
	    progs_string2 = config_prechanged->progs[j];
	 }
	 if (!progs_string1) {
	    break;
	 } else if (progs_string1 == progs_string2) {
	    not_free = TRUE;
	    break;
	 }
      }
 
      if (!not_free) {
	 g_free (progs_string1);
      }

      not_free = FALSE;
   }
}
/* END FIXME!! */


static void
cb_close_prefs_window ()
{
   gint i;

  if (prefs_ok_pressed) {
     strings_free (TRUE);
     memcpy (&conf, config_changed, sizeof(Config));
     conf_apply_to_existing_win (config_prechanged, &conf);
  } else {
     strings_free (FALSE);
     memcpy (&conf, config_prechanged, sizeof(Config));
     conf_apply_to_existing_win (config_changed, &conf);
   }
   g_free (config_changed);
   g_free (config_prechanged);
   config_changed = NULL;
   config_prechanged = NULL;
   prefs_window = NULL;
   prefs_ok_pressed = FALSE;
   prefs_save_config ();

   for (i = 0; i < prefs_pages_num; i++) {
      prefs_pages[i].node = NULL;
      prefs_pages[i].page = NULL;
   }
}


static void
cb_prefs_ok_button ()
{
   prefs_ok_pressed = TRUE;
   gtk_widget_destroy (prefs_window);
}


static void
cb_prefs_apply_button ()
{
   memcpy (&conf, config_changed, sizeof(Config));
   conf_apply_to_existing_win (config_prechanged, &conf);
}


static void
cb_prefs_cancel_button ()
{
   gtk_widget_destroy (prefs_window);
}


static void
cb_mouse_button (GtkWidget *menu_item, gpointer user_data)
{
   gchar **src = user_data, *dest, *tempstr, buf[128], *defval;
   gchar **buttons, **mods;
   gpointer bid_p, mid_p, num_p;
   gint i, j, bid, mid, num, data[4];
   GtkToggleButton *pressed, *released;

   g_return_if_fail (menu_item);
   g_return_if_fail (src && *src && **src);

   bid_p    = gtk_object_get_data (GTK_OBJECT (menu_item), "button-id");
   mid_p    = gtk_object_get_data (GTK_OBJECT (menu_item), "mod-id");
   num_p    = gtk_object_get_data (GTK_OBJECT (menu_item), "num");
   defval   = gtk_object_get_data (GTK_OBJECT (menu_item), "prefs-prechanged");
   pressed  = gtk_object_get_data (GTK_OBJECT (menu_item), "pressed");
   released = gtk_object_get_data (GTK_OBJECT (menu_item), "released");

   bid = GPOINTER_TO_INT (bid_p);
   mid = GPOINTER_TO_INT (mid_p);
   num = GPOINTER_TO_INT (num_p);

   buttons = g_strsplit (*src, ";", 6);
   if (!buttons) return;
   dest = g_strdup ("");

   for (i = 0; i < 6; i++) {
      mods = g_strsplit (buttons[i], ",", 4);
      if (!mods) {
	 g_strfreev (buttons);
	 g_free (dest);
	 return;
      }
      for (j = 0; j < 4; j++) {
	 if (i == bid && j == mid) {
	    if (released && released->active)
	       data[j] = 0 - num;
	    else
	       data[j] = num;
	 } else{
	    g_strstrip (mods[j]);
	    data[j] = atoi (mods[j]);
	 }
      }
      g_snprintf (buf, 128, "%d,%d,%d,%d; ",
		  data[0], data[1], data[2], data[3]);
      tempstr = g_strconcat (dest, buf, NULL);
      g_free (dest);
      dest = tempstr;
      g_strfreev (mods);
   }

   if (defval != *src)
      g_free (*src);

   *src = dest;

   g_strfreev (buttons);
}


static void
cb_mouse_prefs_pressed_radio (GtkWidget *radio, gpointer data)
{
   GtkOptionMenu *option_menu = data;

   g_return_if_fail (option_menu);

   if (option_menu->menu_item)
      gtk_signal_emit_by_name (GTK_OBJECT (option_menu->menu_item),
			       "activate");
}


static void
cb_ctree_select_row (GtkWidget *widget, gint row, gint column,
		     GdkEventButton *event, gpointer data)
{
   GtkCTreeNode *node;
   GtkWidget *page;
   gint num;

   /*
   gtk_clist_get_selection_info (GTK_CLIST (widget),
				 event->x, event->y,
				 &row, NULL);
   */
   node = gtk_ctree_node_nth (GTK_CTREE (widget), row);
   if (!node) return;

   page = gtk_ctree_node_get_row_data (GTK_CTREE (widget), node);

   if (!page || !GTK_IS_WIDGET (page)) return;

   num = gtk_notebook_page_num (GTK_NOTEBOOK (prefs_win.notebook), page);
   if (num >= 0)
      gtk_notebook_set_page (GTK_NOTEBOOK (prefs_win.notebook), num);
}



/*******************************************************************************
 *
 *   Parts of preference window.
 *
 *******************************************************************************/
const gchar *mod_keys[] = {
   N_("Normal"),
   N_("Shift"),
   N_("Control"),
   N_("Mod1")
};
gint mod_keys_num = sizeof (mod_keys) / sizeof (gchar *);



/*******************************************************************************
 *
 *  functions for creating navigate tree
 *
 *******************************************************************************/
static GtkCTreeNode *
navigate_tree_get_parent (PrefsPage *page)
{
   gint i;

   g_return_val_if_fail (page, NULL);
   g_return_val_if_fail (page->title, NULL);

   if (!page->parent) return NULL;

   for (i = 0; i < prefs_pages_num; i++) {
      if (!strcmp (page->parent, prefs_pages[i].title)) {
	 return prefs_pages[i].node;
      }
   }

   return NULL;
}


static GtkWidget *
create_navigate_tree_and_pages ()
{
   GtkWidget *scrolledwin, *ctree;
   GtkWidget *vbox = NULL, *label;
   GtkCTreeNode *node, *parent = NULL;
   Icon *folder, *ofolder;
   gchar *text;
   gint i;

   scrolledwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolledwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_widget_set_usize (scrolledwin, 170, -1);

   ctree = prefs_win.ctree = gtk_ctree_new (1,0);
   gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 0, TRUE);
   gtk_clist_set_selection_mode (GTK_CLIST (ctree), GTK_SELECTION_BROWSE);
   gtk_ctree_set_line_style (GTK_CTREE (ctree), conf.dirview_line_style);
   gtk_ctree_set_expander_style (GTK_CTREE (ctree), conf.dirview_expander_style);
   gtk_clist_set_row_height (GTK_CLIST (ctree), 18);
   gtk_container_add (GTK_CONTAINER (scrolledwin), ctree);
   gtk_widget_show (ctree);

   gtk_signal_connect (GTK_OBJECT (ctree),"select-row",
		       GTK_SIGNAL_FUNC (cb_ctree_select_row), NULL);

   for (i = 0; i < prefs_pages_num; i++) {
      GdkPixmap *pixmap = NULL, *opixmap = NULL;
      GdkBitmap *mask = NULL, *omask = NULL;

      if (!prefs_pages[i].title)
	 continue;

      /* translate page title */
      text = _(prefs_pages[i].title);

      /* create page */
      if (prefs_pages[i].create_page_fn) {
	 vbox = prefs_pages[i].create_page_fn ();
	 label = gtk_label_new (text);
	 gtk_notebook_append_page (GTK_NOTEBOOK(prefs_win.notebook),
				   vbox, label);
      }

      /* get node icon */
      if (prefs_pages[i].icon) {
	 folder  = icon_get (prefs_pages[i].icon);
	 if (folder) {
	    pixmap = folder->pixmap;
	    mask = folder->mask;
	 }
      }
      if (prefs_pages[i].icon_open) {
	 ofolder  = icon_get (prefs_pages[i].icon_open);
	 if (ofolder) {
	    opixmap = ofolder->pixmap;
	    omask = ofolder->mask;
	 }
      }

      if (prefs_pages[i].parent)
	 parent = navigate_tree_get_parent (&prefs_pages[i]);

      /* insert node */
      node = gtk_ctree_insert_node (GTK_CTREE (ctree), parent, NULL,
				    &text, 4,
				    pixmap, mask,
				    opixmap, omask,
				    FALSE, FALSE);
      /*
      if (parent)
	 gtk_ctree_expand (GTK_CTREE (ctree), parent);
      */

      if (vbox) {
	 gtk_ctree_node_set_row_data_full (GTK_CTREE(ctree), node,
					   vbox, NULL);
	 prefs_pages[i].page = vbox;
      }
      prefs_pages[i].node = node;

      vbox = NULL;
      parent = NULL;
   }

   return scrolledwin;
}


GtkWidget *
create_mouse_prefs (const gchar **items,
		    const gchar  *defval,
		    gchar       **dest)
{
   GtkWidget *main_vbox, *vbox, *hbox, *vbox1, *vbox2, *temp_vbox;
   GtkWidget *frame1, *frame_vbox1;
   GtkWidget *label, *notebook, *option_menu, *menu, *menu_item;
   GtkWidget *radio0, *radio1;
   gchar buf[128], **temp, *buttons[6], **mods;
   gint id, num, i, j, k, def;

   g_return_val_if_fail (items, NULL);
   g_return_val_if_fail (defval, NULL);
   g_return_val_if_fail (dest, NULL);

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, TRUE, 0);
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, TRUE, 0);
   vbox2 = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, TRUE, 0);

   temp = g_strsplit (defval, ";", 6);
   if (temp) {
      buttons[0] = temp[1];
      buttons[1] = temp[2];
      buttons[2] = temp[3];
      buttons[3] = temp[0];
      buttons[4] = temp[4];
      buttons[5] = temp[5];
   } else {
      for (i = 0; i < 6; i++)
	 buttons[i] = NULL;
   }

   /* create frames */
   for (i = 0; i < 6; i++) {
      if (i < 3) {
	 num = i + 1;
	 id = num;
      } else if (i == 3) {
	 num = 1;
	 id = 0;
      } else {
	 num = i;
	 id = num;
      }

      if (i / 3 < 1) {
	 temp_vbox = vbox1;
      } else {
	 temp_vbox = vbox2;
      }

      /**********************************************
       * Mouse Button X Frame
       **********************************************/
      if (num == 4)
	 g_snprintf (buf, 128, _("Mouse Button %d (Wheel up)"), num);
      else if (num == 5)
	 g_snprintf (buf, 128, _("Mouse Button %d (Wheel down)"), num);
      else if (i == 3)
	 g_snprintf (buf, 128, _("Mouse Button %d Double Click"), num);
      else
	 g_snprintf (buf, 128, _("Mouse Button %d"), num);
      frame1 = gtk_frame_new (buf);
      gtk_container_set_border_width(GTK_CONTAINER(frame1), 5);
      gtk_box_pack_start(GTK_BOX(temp_vbox), frame1, FALSE, TRUE, 5);
      frame_vbox1 = gtk_vbox_new (FALSE, 0);
      gtk_container_set_border_width(GTK_CONTAINER(frame1), 5);
      gtk_container_add (GTK_CONTAINER (frame1), frame_vbox1);

      notebook = gtk_notebook_new ();
      gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
      gtk_box_pack_start(GTK_BOX(frame_vbox1), notebook, FALSE, FALSE, 5);
      gtk_widget_show (notebook);

      if (buttons[i] && *buttons[i]) {
	 mods = g_strsplit (buttons[i], ",", 4);
      } else {
	 mods = NULL;
      }

      /* create notebook pages */
      for (j = 0; j < mod_keys_num; j++) {
	 label = gtk_label_new (_(mod_keys[j]));
	 gtk_widget_show (label);
	 vbox = gtk_vbox_new (FALSE, 0);
	 gtk_widget_show (vbox);
	 gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
				   vbox, label);

	 if (mods[j] && *mods[j]) {
	    g_strstrip (mods[j]);
	    def = atoi (mods[j]);
	 } else {
	    def = 0;
	 }

	 option_menu = gtk_option_menu_new();
	 menu = gtk_menu_new();

	 gtk_box_pack_start(GTK_BOX(vbox), option_menu, FALSE, FALSE, 5);

	 hbox = gtk_hbox_new (FALSE, 0);
	 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);

	 radio0 = gtk_radio_button_new_with_label (NULL, _("Pressed"));
	 gtk_box_pack_start (GTK_BOX (hbox), radio0, FALSE, FALSE, 0);
	 radio1 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio0),
							       _("Released"));
	 gtk_box_pack_start (GTK_BOX (hbox), radio1, FALSE, FALSE, 0);
	 gtk_signal_connect (GTK_OBJECT (radio0), "toggled",
			     GTK_SIGNAL_FUNC (cb_mouse_prefs_pressed_radio),
			     option_menu);
	 gtk_signal_connect (GTK_OBJECT (radio1), "toggled",
			     GTK_SIGNAL_FUNC (cb_mouse_prefs_pressed_radio),
			     option_menu);

	 if (def < 0)
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
	 else
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio0), TRUE);

	 if (id == 0) {
	    gtk_widget_set_sensitive (radio0, FALSE);
	    gtk_widget_set_sensitive (radio1, FALSE);
	 }

	 /* create option menu items */
	 for (k = 0; items[k]; k++) {
	    menu_item = gtk_menu_item_new_with_label (_(items[k]));
	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "button-id",
				 GINT_TO_POINTER(id));
	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "mod-id",
				 GINT_TO_POINTER(j));
	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "num",
				 GINT_TO_POINTER(k));
	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "prefs-prechanged",
				 (gpointer) defval);
	    gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(cb_mouse_button),
			       dest);

	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "pressed",
				 (gpointer) radio0);
	    gtk_object_set_data (GTK_OBJECT (menu_item),
				 "released",
				 (gpointer) radio1);
	    gtk_menu_append (GTK_MENU(menu), menu_item);
	    gtk_widget_show (menu_item);
	 }
	 gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
	 gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), abs (def));
      }

      g_strfreev (mods);
   }

   g_strfreev (temp);

   return main_vbox;
}


/*******************************************************************************
 *
 *   prefs_open_window:
 *      @ Create preference window. If already opened, raise it and return.
 *
 *   GtkWidget *prefs_window (global valiable):
 *      Pointer to preference window.
 *
 *   Static PrefsWin prefs_win (local valiable):
 *      store pointer to eache widget;
 *
 *******************************************************************************/
void
prefs_open_window ()
{
   GtkWidget *notebook;
   GtkWidget *pane;
   GtkWidget *scrolledwin;
   GtkWidget *button;
   Icon *icon;

   /* if preference window is alredy opend, raise it and return */
   if (prefs_window) {
      gdk_window_raise (prefs_window->window);
      return;
   }

   /* allocate buffer for new config */
   config_prechanged = g_memdup (&conf, sizeof(Config));
   config_changed = g_memdup (&conf, sizeof(Config));

   /* create config window */
   prefs_window = gtk_dialog_new ();
   gtk_window_set_wmclass(GTK_WINDOW(prefs_window), "prefs", "GImageView");
   /* gtk_window_set_default_size (GTK_WINDOW(prefs_window), 600, 400); */
   gtk_window_set_title (GTK_WINDOW (prefs_window), _("Preference")); 
   gtk_signal_connect (GTK_OBJECT(prefs_window), "destroy",
		       GTK_SIGNAL_FUNC(cb_close_prefs_window), NULL);

   /* pane */
   pane = gedo_hpaned_new ();
   gtk_container_set_border_width (GTK_CONTAINER (pane), 5);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->vbox), 
		       pane, TRUE, TRUE, 0);

   notebook = prefs_win.notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
   gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);

   scrolledwin = create_navigate_tree_and_pages ();

   gedo_paned_add1 (GEDO_PANED (pane), scrolledwin);
   gedo_paned_add2 (GEDO_PANED (pane), notebook);

   /* initialize sensitive */
   prefs_ui_common_set_sensitive ();
   prefs_ui_imagewin_set_sensitive ();
   prefs_ui_tumbwin_set_sensitive ();
   prefs_plugin_set_sensitive (FALSE);
   prefs_ui_progs_set_sensitive ();
   prefs_ui_etc_set_sensitive ();

   /* dialog buttons */
   button = gtk_button_new_with_label (_("OK"));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->action_area), 
		       button, TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (cb_prefs_ok_button),
			      GTK_OBJECT (prefs_window));
   GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);
 
   gtk_widget_grab_focus (button);

   button = gtk_button_new_with_label (_("Apply"));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->action_area), 
		       button, TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (cb_prefs_apply_button),
			      GTK_OBJECT (prefs_window));
   GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);

   button = gtk_button_new_with_label (_("Cancel"));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->action_area), 
		       button, TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (cb_prefs_cancel_button),
			      GTK_OBJECT (prefs_window));
   GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);

   gtk_widget_show_all (prefs_window);

   icon = icon_get ("prefs");
   if (icon)
      gdk_window_set_icon(prefs_window->window, NULL,
			  icon->pixmap, icon->mask);
}


gboolean
prefs_win_query ()
{
   if (prefs_window)
      return TRUE;
   else
      return FALSE;
}
